#!/usr/bin/env python3
# __main__.py

"""Awesome Check Links 

Usage:  
    aclinks [--verbose --exit --down] -f FILE
    aclinks (-h | --help | --version)

Options:
    -f --file       Markdown file to scan.
    -e --exit       Stop your build pipeline if a site is down
    -d --down       Show only down links and their line number.
    -v --verbose    Verbose mode. Print more to stdout.
    -h --help       Show this help.
    --version       Show version.

"""

from aclinks import checker
from docopt import docopt 
 

if __name__ == "__main__":
    arguments = docopt(__doc__, version='Awesome check links')

    ########## CLI VAR ##########

    __verbose = arguments["--verbose"]
    __down = arguments["--down"]
    __exit = arguments["--exit"]

    __file = arguments["FILE"]
    __file_ext = __file.split(".")[-1]

    if __file_ext != "md":
        print("aclinks take only md file ! ")
        exit()

    links = checker.extract_links(__file)
    

    if __down:
        down_links = checker.get_down_links(__file, links,__verbose, __exit)        
    
    else : 
        checker.get_all_status(__file, links,__verbose, __exit)


    


import re 
import os
import requests


def extract_links(file):
    """Extract all links in the md file
    Args:
        file (str) : The file to scan 
    Return:
        list. List of all extracted links and their line number. 
        Data Strucure : 
            [(line_0, matchObject_0) , ... , (line_n, matchObject_n)]
        
        For matchOject look at python regex documentation : https://docs.python.org/fr/3.6/library/re.html#re.regex.search

    """

    found = []
    pattern = re.compile('\[(.+)\]\(([^ ]+)\)')

    for i, line in enumerate(open(file)):
        for match in re.finditer(pattern, line):
            if match.group(2).startswith("http"): # only select link which start with 'http'
                found.append(
                    (
                        i+1, # line 
                        match
                    )
                )
    return found

def get_down_links(file, links, __verbose, __exit):
    """
    Args: 
        links (list) : List of links and their line number
    
    Return:
        list. List of down website 
    
    """
    down = []

    for l in links :
        r = requests.head(l[1][2])
        if r.status_code != 200:
            down.append(
                (
                    l[0],
                    l[1][2]
                )
            )
            __verbose and print("At line", l[0] , ":" , l[1][2], ":", r.reason, "(", r.status_code, ")")
            if __exit:
                r.raise_for_status()
    return down


# For next version
def remove_down_links(file, line_numbers):
    """In a file, delete the lines at line number in given list"""
    is_skipped = False
    counter = 0
    # Create name of dummy / temporary file
    dummy_file = file + '.bak'
    # Open original file in read only mode and dummy file in write mode
    with open(file, 'r') as read_obj, open(dummy_file, 'w') as write_obj:
        # Line by line copy data from original file to dummy file
        for line in read_obj:
            # If current line number exist in list then skip copying that line
            if counter not in line_numbers:
                write_obj.write(line)
            else:
                is_skipped = True
            counter += 1
 
    # If any line is skipped then rename dummy file as original file
    if is_skipped:
        os.remove(file)
        os.rename(dummy_file, file)
    else:
        os.remove(dummy_file)


def get_all_status(file, links, __verbose, __exit):
    """Get status for all links of the md file 
    Args:
        file (str) : The file to scan.
        links (list) : List of links to check for status.
        __verbose : Add more to output 
    
    Return: 
        list. list of line number, links, and status.
        Data Strucure : 
            [(line_0, link_0, status_0) , ... , (line_n, link_n, status_n)]
        

    """
    link_data = []

    for l in links :
        r = requests.head(l[1][2])
        link_data.append(
            (
                l[0],
                l[1][2],
                r.reason
            )
        )
        __verbose and print("At line", l[0], ":", l[1][2], ":", r.reason, "(", r.status_code, ")" )
        if __exit:
            r.raise_for_status()
    return link_data




import os
from setuptools import setup

from aclinks import __version__

# Utility function to read the README file.
# Used for the long_description.  It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

def read_requirements(filename):
    with open(filename) as f:
        return f.read().splitlines()

settings = dict(
    name = 'awesome-check-link',
    packages = ['aclinks'],
    version = __version__,
    author = 'shadawck',
    author_email = 'hug211mire@gmail.com',
    description = ('Check if links in md file and more particuraly in awesome-list are down or not.'),
    license = 'MIT',
    keywords = 'awesome-check-link, awesome, awesome list, checker, down, link checker',
    url = 'https://github.com/remiflavien1/awesome-check-link',
    long_description=read('README.md'),
    long_description_content_type='text/markdown',
    install_requires=read_requirements('requirements.txt'),
    tests_require=read_requirements('test-requirements.txt'),
    classifiers=[
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'License :: OSI Approved :: MIT License',
    ],
    entry_points={
        "console_scripts": [
            "aclinks=aclinks.__main__:main",
        ]
    },
)

setup(**settings)
